#!/usr/bin/env python
# -*- coding: UTF-8 -*-


import os


from dbpedia.lookup.bp import LookupOrchestrator

from baseblock import EnvIO


def main():

    entity_type = EnvIO.str_or_exception('ENTITY_TYPE')
    entity_name = EnvIO.str_or_exception('ENTITY_NAME')
    entity_label = EnvIO.str_or_exception('ENTITY_LABEL')

    bp = LookupOrchestrator()
    bp.process(entity_type=entity_type,
               entity_name=entity_name,
               entity_label=entity_label)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" API for dbPedia Recipes """

from baseblock import EnvIO
from baseblock import Stopwatch
from baseblock import BaseObject

from dbpedia_get.lookup.dmo import PageCacheReader

from dbpedia_get.lookup.bp import LookupOrchestrator


class RecipesAPI(BaseObject):
    """ API for dbPedia Recipes """

    def __init__(self):
        """ Change Log

        Created:
            14-Jul-2022
            craigtrim@gmail.com
            *   https://github.com/craigtrim/dbpedia/issues/3
        """
        BaseObject.__init__(self, __name__)
        self._lookup = LookupOrchestrator().process

    def _download_named_link(self,
                             link_name: str) -> dict:
        """ Download the dbPedia page corresponding to a named link

        Args:
            link_name (str): the link name

        Returns:
            dict: the corresponding dbPedia page
        """
        entity_type = link_name
        entity_name = link_name.replace(' ', '_').lower()
        entity_label = link_name

        return self._lookup(
            entity_type=entity_type,
            entity_name=entity_name,
            entity_label=entity_label)

    def download_all_links(self) -> None:
        """ Download the Links for all Cached Pages """
        d_results = PageCacheReader().process()

        for key in d_results:
            [self._download_named_link(x) for x in d_results[key]['links']]

    def download_links(self,
                       entity_type: str,
                       entity_name: str,
                       entity_label: str) -> int:
        """ Download the Links for a given Entity

        Args:
            entity_type (str): the type of entity
                e.g.,   the object in an rdfs:subClass relationship
            entity_name (str): the name of the entity
                e.g.,   the subject of an rdf:type relationship
            entity_label (str): the human readable label for the entity
                e.g.,   the object of an rdfs:label relationship

        Returns:
            int: the number of links accessed
        """
        d_page = self._lookup(
            entity_type=entity_type,
            entity_name=entity_name,
            entity_label=entity_label)

        if 'links' not in d_page:
            return 0

        [self._download_named_link(x) for x in d_page['links']]

        return len(d_page['links'])



#!/usr/bin/env python
# -*- coding: UTF-8 -*-


import os
import json

from baseblock import FileIO
from baseblock import BaseObject

from dbpedia.lookup.dto import WikipediaPageStatus


class PageCacheIO(BaseObject):
    """ I/O Operations for a DBpedia page from disk (cache) """

    def __init__(self):
        """ Change Log

        Created:
            14-Jul-2022
            craigtrim@gmail.com
            *   https://github.com/craigtrim/dbpedia/issues/1
        """
        BaseObject.__init__(self, __name__)

    def read(self,
             title_path: str) -> dict or None:

        if not os.path.exists(title_path):
            if self.isEnabledForDebug:
                self.logger.debug('\n'.join([
                    "DBpedia Article Not Cached",
                    f"\tPath: {title_path}"]))
            return None

        if self.isEnabledForDebug:
            self.logger.debug('\n'.join([
                "Reading DBpedia Article from File",
                f"\tPath: {title_path}"]))

        return FileIO.read_json(title_path)

    def write_fail(self,
                   title_path: str,
                   entity_type: str,
                   entity_name: str,
                   entity_label: str,
                   status: WikipediaPageStatus) -> str:
        """ This method writes an empty file to the cache
            This prevents the system from continually requesting a page that does not exist

        Args:
            title_path (str): the path of the local copy
            entity_type (str): the type of entity
                e.g.,   the object in an rdfs:subClass relationship
            entity_name (str): the name of the entity
                e.g.,   the subject of an rdf:type relationship
            entity_label (str): the human readable label for the entity
                e.g.,   the object of an rdfs:label relationship
            status (WikipediaPageStatus): the status of the request

        Returns:
            str: the title path
        """

        d_page = {
            "is_empty": True,
            "status": status.name,
            "entity": {
                "type": entity_type,
                "name": entity_name,
                "label": entity_label
            }
        }

        FileIO.write_json(data=d_page,
                          file_path=title_path)

        if self.isEnabledForInfo:
            self.logger.info('\n'.join([
                "Wrote Empty Entity to File",
                f"\tPath: {title_path}"]))

        return title_path

    def write(self,
              d_page: dict,
              title_path: str) -> str:

        if self.isEnabledForInfo:
            self.logger.info('\n'.join([
                "Caching DBpedia Article to File",
                f"\tPath: {title_path}"]))

        FileIO.write_json(data=d_page,
                          file_path=title_path)

        return title_path



#!/usr/bin/env python
# -*- coding: UTF-8 -*-


from typing import Callable

from baseblock import Enforcer
from baseblock import BaseObject


class DBPediaContentNormalizer(BaseObject):
    """ Normalize the DBPedia Content Section """

    def __init__(self,
                 segmenter: Callable):
        """ Change Log

        Created:
            14-Jul-2022
            craigtrim@gmail.com
            *   https://github.com/craigtrim/dbpedia/issues/1
        """
        BaseObject.__init__(self, __name__)
        self._segmenter = segmenter

    def _process(self,
                 content: list) -> list:

        def _split() -> list:
            master = []

            for item in content:
                for line in item.split('\n'):
                    master.append(line)

            master = [line.strip() for line in master if line]
            master = [line for line in master if line and len(line)]

            return master

        def _sentencize(lines: list) -> list:

            master = []
            for line in lines:
                while '  ' in line:
                    line = line.replace('  ', ' ')

                [master.append(x)
                 for x in self._segmenter(input_text=line)]

            master = [line.strip() for line in master if line]
            master = [line for line in master if line and len(line)]

            return master

        return _sentencize(_split())

    def process(self,
                content: list) -> list:
        return self._process(content)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" for a given input find the associated dbPedia entry """


from typing import Callable

from baseblock import BaseObject

from dbpedia.lookup.dmo import DBpediaPageFinder
from dbpedia.lookup.dmo import PageCacheIO
from dbpedia.lookup.dmo import PageTitlePath
from dbpedia.lookup.dto import WikipediaPageStatus


class PerformPageLookup(BaseObject):
    """ for a given input find the associated dbPedia entry """

    def __init__(self,
                 auto_suggest: bool,
                 transform_page: Callable):
        """ Change Log

        Created:
            14-Jul-2022
            craigtrim@gmail.com
            *   https://github.com/craigtrim/dbpedia/issues/1

        Args:
            auto_suggest (bool): _description_
        """
        BaseObject.__init__(self, __name__)
        self._transform_page = transform_page
        self._pathr = PageTitlePath().process
        self._finder = DBpediaPageFinder(auto_suggest)

        cache = PageCacheIO()
        self._read = cache.read
        self._write = cache.write
        self._write_fail = cache.write_fail

    def process(self,
                entity_type: str,
                entity_name: str,
                entity_label: str) -> tuple:
        """ Lookup a dbPedia Page for the provided information

        Args:
            entity_type (str): the type of entity
                e.g.,   the object in an rdfs:subClass relationship
            entity_name (str): the name of the entity
                e.g.,   the subject of an rdf:type relationship
            entity_label (str): the human readable label for the entity
                e.g.,   the object of an rdfs:label relationship

        Sample Usage:
            to look up the dbPedia page for 'network topology' use the following parameters:
                bp.process(
                    entity_type='topology',
                    entity_name='network_topology',
                    entity_label='network topology')

            The most important parameter is entity_name; but entity_type and entity_label are both useful
                when there is a need to differentiate an ambigous entity_name
                e.g.,   if you want the dbpedia page for the 'pandas' software package
                        an entity_type of 'software' will help you differentiate from pandas as an animal

        Returns:
            dict: _description_
        """

        title_path = self._pathr(
            title=entity_name,
            make_dirs=True)

        d_page = self._read(title_path)
        if d_page is not None and "content" in d_page and len(d_page["content"]):
            return d_page, WikipediaPageStatus.FROM_CACHE

        w_page, status = self._finder.process(entity_label)
        if not w_page:
            self._write_fail(status=status,
                             title_path=title_path,
                             entity_type=entity_type,
                             entity_name=entity_name,
                             entity_label=entity_label)
            return None, status

        d_page = self._transform_page(page=w_page,
                                      title=entity_name)
        if d_page is None or not len(d_page):
            return None, WikipediaPageStatus.FAIL_TRANSFORM

        d_page["entity"] = {
            "type": entity_type,
            "name": entity_name,
            "label": entity_label
        }

        self._write(d_page=d_page,
                    title_path=title_path)

        return d_page, status



from .dbpedia_content_normalizer import DBPediaContentNormalizer
from .dbpedia_page_finder import DBpediaPageFinder
from .page_cache_io import PageCacheIO
from .page_cache_entry import PageCacheEntry
from .page_cache_reader import PageCacheReader
from .page_title_path import PageTitlePath
from .spacy_doc_segmenter import SpacyDocSegmenter



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" dbPedia Orchestrator for dbPedia Redirects """

import os

from baseblock import EnvIO
from baseblock import BaseObject

from dbpedia.redirects.svc import SearchRedirectsFile
from dbpedia.redirects.svc import GenerateRedirectsFile


class RedirectsOrchestrator(BaseObject):
    """ dbPedia Orchestrator for dbPedia Redirects """

    def __init__(self):
        """ Change Log

        Created:
            14-Jul-2022
            craigtrim@gmail.com
            *   https://github.com/craigtrim/dbpedia/issues/1
        """
        BaseObject.__init__(self, __name__)

    def generate(self) -> dict:
        GenerateRedirectsFile().process()

    def search(self) -> dict:
        input_text = EnvIO.str_or_exception('REDIRECTS_SEARCH_TERM')
        SearchRedirectsFile().process(input_text)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-


import os
import time

from wikipedia import WikipediaPage

from baseblock import BaseObject

from dbpedia.lookup.dmo import PageCacheIO
from dbpedia.lookup.dmo import PageTitlePath
from dbpedia.lookup.dmo import PageCacheReader
from dbpedia.lookup.dmo import DBpediaPageFinder
from dbpedia.lookup.svc import PerformPageTransform


class FollowPageLinks(BaseObject):
    """ For a given page, download all the linked pages """

    def __init__(self):
        """ Change Log

        Created:
            14-Jul-2022
            craigtrim@gmail.com
            *   https://github.com/craigtrim/dbpedia/issues/1
        """
        BaseObject.__init__(self, __name__)

        auto_suggest = os.environ['AUTO_SUGGEST'].lower() == 'true'

        self._cache = PageCacheIO()
        self._pathr = PageTitlePath().process
        self._finder = DBpediaPageFinder(auto_suggest)

    @staticmethod
    def _page_exists(d_page: dict) -> bool:
        if d_page is not None and "content" in d_page and len(d_page["content"]):
            return d_page

    def _page(self,
              entity_name: str) -> dict or None:
        d_page = self._cache.read(entity_name)
        if self._page_exists(d_page):
            return d_page

        self.logger.error('\n'.join([
            "Entity Page Not Found",
            f"\tEntity Name: {entity_name}"]))

        raise ValueError

    def _links(self,
               d_page: dict) -> dict:

        if "links" not in d_page:  # normal; empty page
            if self.isEnabledForInfo:
                self.logger.info('\n'.join([
                    "Empty Page",
                    "Entity Was Not Found in WikiPedia",
                    f"\tEntity Name: {d_page['entity']['name']}"]))
            return []

        links = d_page['links']

        links = [link for link in links
                 if 'list of' not in link.lower()]

        d_links = {link: self._cache.read(link)
                   for link in links}

        d_links = {link: d_links[link]
                   for link in d_links if d_links[link] is None}

        if self.isEnabledForDebug:
            self.logger.debug('\n'.join([
                "Retrieved Cached Page",
                f"\tEntity Name: {d_page['entity']['name']}",
                f"\tTotal Links: {len(links)}",
                f"\tTotal Uncached Links: {len(d_links)}"]))

        return d_links

    def _master_page(self,
                     entity_name: str) -> dict:
        if entity_name == "random":
            return PageCacheReader().random()

        return self._page(self._pathr(entity_name))

    def process(self,
                entity_type: str,
                entity_name: str,
                entity_label: str) -> tuple:

        d_master_page = self._master_page(entity_name)
        d_links = self._links(d_master_page)

        for link in d_links:
            title_path = self._pathr(title=link,
                                     make_dirs=True)
            w_page, status = self._finder.process(link)

            if not w_page:
                # we are spidering known Wikipedia Links,
                # so this should not happen (but it does...)
                self._cache.write_fail(status=status,
                                       title_path=title_path,
                                       entity_type=entity_type,
                                       entity_name=entity_name,
                                       entity_label=entity_label)
                if self.isEnabledForWarning:
                    self.logger.warning('\n'.join([
                        "Page Not Found",
                        f"\tEntity Name: {link}"]))
                continue

            svc_transform = PerformPageTransform(title=link,
                                                 page=w_page)

            d_page = svc_transform.process()
            if d_page is None or not len(d_page):
                self._cache.write_fail(status=status,
                                       title_path=title_path,
                                       entity_type=entity_type,
                                       entity_name=entity_name,
                                       entity_label=entity_label)
                if self.isEnabledForWarning:
                    self.logger.warning('\n'.join([
                        "Page Transformation Failed",
                        f"\tEntity Name: {link}"]))
                continue

            # we have no entity information;
            # these pages exist outside the CL KB
            d_page["entity"] = {
                "type": None,
                "name": None,
                "label": None}

            self._cache.write(d_page=d_page,
                              title_path=title_path)

            # persist twice if the link is actually a redirect to the primary page
            if link != d_page["title"]:
                self._cache.write(d_page=d_page,
                                  title_path=title_path)

        if entity_name == "random":
            self.process(entity_type="*",
                         entity_name="random",
                         entity_label="*")  # infinite recursion



#!/usr/bin/env python
# -*- coding: UTF-8 -*-


from baseblock import EnvIO
from baseblock import Stopwatch
from baseblock import BaseObject

from dbpedia_get.lookup.dmo import PageCacheReader

from dbpedia_get.lookup.svc import PerformPageTransform
from dbpedia_get.lookup.svc import PerformPageLookup


class LookupOrchestrator(BaseObject):
    """ dbPedia Orchestrator for Wikipedia Lookup """

    def __init__(self):
        """ Change Log

        Created:
            14-Jul-2022
            craigtrim@gmail.com
            *   https://github.com/craigtrim/dbpedia/issues/1
        """
        BaseObject.__init__(self, __name__)
        auto_suggest = EnvIO.exists_as_true('AUTO_SUGGEST')
        self._total_terms = EnvIO.int_or_exception('TOTAL_TERMS')

        self._find_page = PerformPageLookup(
            auto_suggest=auto_suggest,
            transform_page=PerformPageTransform().process).process

    def process(self,
                entity_type: str,
                entity_name: str,
                entity_label: str) -> dict:
        """ Lookup a dbPedia Page for the provided information

        Args:
            entity_type (str): the type of entity
                e.g.,   the object in an rdfs:subClass relationship
            entity_name (str): the name of the entity
                e.g.,   the subject of an rdf:type relationship
            entity_label (str): the human readable label for the entity
                e.g.,   the object of an rdfs:label relationship

        Sample Usage:
            to look up the dbPedia page for 'network topology' use the following parameters:
                bp.process(
                    entity_type='topology',
                    entity_name='network_topology',
                    entity_label='network topology')

            The most important parameter is entity_name; but entity_type and entity_label are both useful
                when there is a need to differentiate an ambigous entity_name
                e.g.,   if you want the dbpedia page for the 'pandas' software package
                        an entity_type of 'software' will help you differentiate from pandas as an animal

        Returns:
            dict: the JSON rendering of the dbPedia page
        """

        sw = Stopwatch()

        d_results = PageCacheReader().process()

        if entity_name in d_results:
            if self.isEnabledForDebug:
                self.logger.debug('\n'.join([
                    "Entity Exists in Cache",
                    f"\tEntity Type: {entity_type}",
                    f"\tEntity Name: {entity_name}",
                    f"\tEntity Label: {entity_label}"]))
            return d_results[entity_name]

        d_page, status = self._find_page(
            entity_type=entity_type,
            entity_name=entity_name,
            entity_label=entity_label)

        if self.isEnabledForInfo:
            self.logger.info('\n'.join([
                "Lookup Orchestration Completed",
                f"\tTotal Time: {str(sw)}",
                f"\tStatus: {status}",
                f"\tEntity Type: {entity_type}",
                f"\tEntity Name: {entity_name}",
                f"\tEntity Label: {entity_label}"]))

        return d_page



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Search Redirects file for a given Term """

import os

from baseblock import BaseObject

from dbpedia.redirects.dmo import DbPediaRedirectsLookup


class SearchRedirectsFile(BaseObject):
    """ Search Redirects file for a given Term """

    def __init__(self):
        """ Change Log

        Created:
            14-Jul-2022
            craigtrim@gmail.com
            *   https://github.com/craigtrim/dbpedia/issues/1
        """
        BaseObject.__init__(self, __name__)
        self._lookup = DbPediaRedirectsLookup()

    def process(self,
                input_text: str) -> None:

        self._lookup.process(input_text)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-


import os
from random import randint
from json.decoder import JSONDecodeError

from baseblock import BaseObject
from baseblock import FileIO
from baseblock import Stopwatch

from dbpedia.lookup.dmo import PageCacheEntry


class PageCacheReader(BaseObject):
    """ Read and Summarize the entire Page Cache """

    def __init__(self):
        """ Change Log

        Created:
            14-Jul-2022
            craigtrim@gmail.com
            *   https://github.com/craigtrim/dbpedia/issues/1
        """
        BaseObject.__init__(self, __name__)
        self._cache_path = self._get_cache_path()

    def _read_cached_file(self,
                          file_path: str) -> dict or None:
        try:
            return FileIO.read_json(file_path)
        except JSONDecodeError as e:
            if self.isEnabledForWarning:
                self.logger.warning('\n'.join([
                    "Unable to Read Cached File",
                    f"\tFile Path: {file_path}"]))
            self.logger.exception(e)

    @staticmethod
    def _get_cache_path() -> str:
        """ Get the Cache Path and Create the Cache Directory if it does not exist

        Returns:
            str: the absolute path to the cached directory
        """
        cache_path = os.path.normpath(
            os.path.join(
                os.environ["PROJECT_BASE"],
                "resources/cache",
                os.environ["CACHE_NAME"]))

        FileIO.exists_or_create(cache_path)

        return cache_path

    def _walk_cache(self) -> list:
        """ Walk through Cache and Load File Paths 

        Returns:
            list: list of absolute paths to cached files
        """
        file_paths = []
        for subdir, _, files in os.walk(self._cache_path):
            for f in files:
                file_paths.append(os.path.join(
                    os.environ["PROJECT_BASE"], subdir, f))
        return file_paths

    def random(self,  # TODO: I think this should go in a new component ... ?
               number_of_attempts: int = 1000) -> dict:
        file_paths = self._walk_cache()

        for i in range(number_of_attempts):
            file_path = file_paths[randint(0, len(file_paths))]

            d_page = self._read_cached_file(file_path)
            if not d_page:
                continue
            if 'is_empty' in d_page and d_page['is_empty'] is True:
                continue

            return d_page

    def process(self) -> dict:
        sw = Stopwatch()

        d_results = {}
        file_paths = self._walk_cache()

        for file_path in file_paths:
            d_page = self._read_cached_file(file_path)
            if d_page:
                d_results[d_page["entity"]["name"]] = d_page

        if self.isEnabledForDebug:
            self.logger.debug('\n'.join([
                "Page Cache Reader Completed",
                f"\tTotal Results: {len(d_results)}",
                f"\tTotal Time: {str(sw)}"]))

        return d_results



#!/usr/bin/env python
# -*- coding: utf-8 -*-


import pandas as pd

from baseblock import EnvIO
from baseblock import Stopwatch
from baseblock import BaseObject


class DbPediaRedirectsLookup(BaseObject):
    """ Given a dbPedia title, lookup the list of dbPedia Redirects """

    def __init__(self):
        """ Change Log

        Created:
            14-Jul-2022
            craigtrim@gmail.com
            *   https://github.com/craigtrim/dbpedia/issues/1
        """
        BaseObject.__init__(self, __name__)
        sw = Stopwatch()
        file_path = EnvIO.str_or_exception('REDIRECTS_FILE')
        self._df_redirects = pd.read_csv(file_path,
                                         sep="\t",
                                         encoding="utf-8")

        if self.isEnabledForInfo:
            self.logger.info('\n'.join([
                "Finished Loading Redirects File",
                f"\tTotal Time: {str(sw)}"]))

    def _lookup(self,
                input_text: str) -> list:

        canon = input_text.lower().strip()
        if '_' in canon:
            canon = canon.replace('_', ' ')

        df = self._df_redirects
        df2 = df[df['Canon'] == canon]

        def _variants() -> list:
            if df2.empty:
                return []
            s = set(df2['Variant'].unique())
            s = [x for x in s if type(x) == str]
            return sorted(s)

        variants = _variants()
        variants = [x.replace('_', ' ') for x in variants
                    if '(' not in x]

        if self.isEnabledForDebug:
            self.logger.debug('\n'.join([
                "Located Variations",
                f"\tCanon: {canon}",
                f"\tTotal Variants: {len(variants)}",
                f"\tVariants: {variants}"]))

        return variants

    def process(self,
                input_text: str) -> list:
        return self._lookup(input_text)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-


from enum import Enum


class WikipediaPageStatus(Enum):
    PASS = 100
    FROM_CACHE = 110
    FAIL_INVALID_INPUT = 400
    FAIL_PAGE_ERROR = 410
    FAIL_KEY_ERROR = 420
    FAIL_DISAMBIGUATION_ERROR = 430
    FAIL_WIKIPEDIA_ERROR = 440
    FAIL_OS_ERROR = 450
    FAIL_TRANSFORM = 460
    FAIL_OTHER = 500



#!/usr/bin/env python
# -*- coding: UTF-8 -*-


import os

from baseblock import BaseObject


class PageTitlePath(BaseObject):
    """ I/O Operations for a DBpedia page from disk (cache) """

    def __init__(self):
        """ Change Log

        Created:
            14-Jul-2022
            craigtrim@gmail.com
            *   https://github.com/craigtrim/dbpedia/issues/1
        """
        BaseObject.__init__(self, __name__)
        self._cache_name = os.environ['CACHE_NAME']

    @staticmethod
    def _file_title(title: str) -> str:
        title = title.replace(' ', '_').lower().strip()

        for ch in ['/', '|', '@', '#', '$', '%', '^', '&',
                   '*', '(', ')', '[', ']', '{', '}', '\\', ':']:
            if ch in title:
                title = title.replace(ch, '_').strip()

        for ch in ['"', "'", '?', '!']:
            if ch in title:
                title = title.replace(ch, '').strip()

        while '__' in title:
            title = title.replace('__', '_')

        return title

    def process(self,
                title: str,
                make_dirs: bool = False) -> str:

        file_title = self._file_title(title)

        def _cleanse_path(a_ch: str) -> str:
            if a_ch is None:
                return "_"
            if a_ch.isalpha():
                return a_ch
            return "_"

        def _ch0() -> str:
            if len(file_title) > 0:
                return _cleanse_path(file_title[0])
            return "_"

        def _ch1() -> str:
            if len(file_title) > 1:
                return _cleanse_path(file_title[1])
            return "_"

        def _ch2() -> str:
            if len(file_title) > 2:
                return _cleanse_path(file_title[2])
            return "_"

        rel_path = f"resources/cache/{self._cache_name}\\{_ch0()}\\{_ch1()}\\{_ch2()}"

        abs_path = os.path.normpath(
            os.path.join(os.environ["PROJECT_BASE"],
                         rel_path))

        if make_dirs and not os.path.exists(abs_path):
            os.makedirs(abs_path)

        return os.path.join(abs_path,
                            f"{file_title}.json")



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Parse the large Redirects file downloaded from dbPedia's site """


import pandas as pd
from pandas import DataFrame

from tabulate import tabulate

from baseblock import BaseObject
from baseblock import Stopwatch


class DBpediaRedirectsParser(BaseObject):
    """ Parse the large Redirects file downloaded from dbPedia's site """

    def __init__(self,
                 input_file: str):
        """ Change Log

        Created:
            14-Jul-2022
            craigtrim@gmail.com
            *   https://github.com/craigtrim/dbpedia/issues/1

        Args:
            input_file (str): _description_
        """
        BaseObject.__init__(self, __name__)
        self._input_file = input_file

    @staticmethod
    def _is_valid(input_text: str or None) -> bool:
        if not input_text or input_text is None or not len(input_text):
            return False
        # if '%' in input_text:
        #     return False
        # if '(' in input_text or ')' in input_text:
        #     return False
        return input_text.replace('_', '').isalpha()

    def _to_list(self) -> list:
        """ Sample Input
            <http://dbpedia.org/resource/Astable> <http://dbpedia.org/ontology/wikiPageRedirects> <http://dbpedia.org/resource/Multivibrator> .
        """
        results = []

        def read_file():
            """
            Process the file line by line using the file's returned iterator
            """
            try:
                with open(self._input_file) as file_handler:
                    while True:
                        yield next(file_handler)
            except (IOError, OSError):
                self.logger.error('\n'.join([
                    "Error opening / processing file",
                    f"\tInput File: {self._input_file}"]))
            except StopIteration:
                pass

        ctr = 0
        sw = Stopwatch()

        for line in read_file():

            tokens = [x.split('/')[-1].split('>')[0].strip()
                      for x in line.split(' ')]

            tokens = [tokens[0], tokens[2]]
            tokens = [x.strip() for x in tokens]

            if len(tokens) != 2:
                continue

            variant = tokens[0].strip().lower()
            if not self._is_valid(variant):
                continue

            canon = tokens[1].strip().lower()
            if not self._is_valid(canon):
                continue

            canon = canon.replace('_', ' ')
            variant = variant.replace('_', ' ')

            if canon == variant:
                continue

            results.append({"Canon": canon,
                            "Variant": variant})

            ctr += 1
            if ctr % 1000000 == 0:
                self.logger.debug(f"Status {ctr} in {str(sw)}")
                sw = Stopwatch()

        return results

    def _to_dataframe(self,
                      results: list) -> DataFrame:
        sw = Stopwatch()
        results = self._to_list()

        df = pd.DataFrame(results)
        df = df.sort_values(by=['Canon'], ascending=True)

        if self.isEnabledForDebug:
            self.logger.debug('\n'.join([
                "Redirects File Parsing Completed",
                f"\tTotal Size: {len(df)}",
                f"\tTotal Time: {str(sw)}",
                tabulate(df.sample(5),
                         tablefmt='psql',
                         headers='keys')]))

        return df

    def process(self) -> DataFrame:
        return self._to_dataframe(self._to_list())



#!/usr/bin/env python
# -*- coding: UTF-8 -*-


from Levenshtein import distance as lev

from baseblock import BaseObject

from dbpedia.lookup.dto import WikipediaPageStatus


class PageCacheEntry(BaseObject):
    """ Create an Entry in the Page Cache """

    def __init__(self):
        """ Change Log

        Created:
            14-Jul-2022
            craigtrim@gmail.com
            *   https://github.com/craigtrim/dbpedia/issues/1
        """
        BaseObject.__init__(self, __name__)

    @staticmethod
    def _summary(d_page: dict,
                 threshold: int = 100) -> str:
        tokens = []
        for sentence in d_page['summary']:

            for sentoken in sentence.split(' '):
                tokens.append(sentoken)

                if len(''.join(tokens)) > threshold:
                    return f"{' '.join(tokens)} ..."

        return f"{' '.join(tokens)} ..."

    @staticmethod
    def _lev_ratio(entity_label: str,
                   title: str) -> int:
        """ Purpose
        Derive the Levenstein Ratio between 
        Entity Label and Wikipedia Entry Title """

        title = title.lower().replace('_', ' ')
        entity_label = entity_label.lower().replace('_', ' ')

        lev_ratio = lev.ratio(entity_label, title)

        if lev_ratio == 1:
            return 100
        if lev_ratio == 0:
            return 0

        return int(round(lev_ratio * 100, 0))

    def process(self,
                d_page: dict,
                status: WikipediaPageStatus) -> dict:

        def title() -> str or None:
            if d_page:
                return d_page['title']

        title = title()

        def summary() -> str or None:
            if d_page:
                return self._summary(d_page)

        def url() -> str or None:
            if d_page:
                return d_page["url"]

        def ratio() -> str:
            if d_page:
                return self._lev_ratio(title=title,
                                       entity_label=d_page["entity"]["label"])
            return 0

        return {
            "URL": url(),
            "Title": title,
            "Ratio": ratio(),
            "Summary": summary(),
            "Status": status.name,
            "entity": {
                "type": d_page["entity"]["type"],
                "name": d_page["entity"]["name"],
                "label": d_page["entity"]["label"],
            },
        }



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['dbpedia_get',
 'dbpedia_get.lookup',
 'dbpedia_get.lookup.bp',
 'dbpedia_get.lookup.dmo',
 'dbpedia_get.lookup.dto',
 'dbpedia_get.lookup.svc',
 'dbpedia_get.recipes',
 'dbpedia_get.recipes.bp',
 'dbpedia_get.recipes.dmo',
 'dbpedia_get.recipes.svc',
 'dbpedia_get.redirects',
 'dbpedia_get.redirects.bp',
 'dbpedia_get.redirects.dmo',
 'dbpedia_get.redirects.svc',
 'dbpedia_get.transform',
 'dbpedia_get.transform.bp',
 'dbpedia_get.transform.dmo',
 'dbpedia_get.transform.svc']

package_data = \
{'': ['*']}

install_requires = \
['Unidecode',
 'apache-beam>=2.40.0,<3.0.0',
 'baseblock',
 'datasets>=2.4.0,<3.0.0',
 'ipykernel>=6.15.1,<7.0.0',
 'jupyter>=1.0.0,<2.0.0',
 'mwparserfromhell>=0.6.4,<0.7.0',
 'openpyxl>=3.0.7,<4.0.0',
 'pandas>=1.2.3,<2.0.0',
 'python-Levenshtein>=0.12.2,<0.13.0',
 'spacy==3.3',
 'tabulate',
 'wikipedia>=1.4.0,<2.0.0']

setup_kwargs = {
    'name': 'dbpedia-get',
    'version': '0.1.1',
    'description': 'dbPedia Concept Linking and Redirect Analysis',
    'long_description': None,
    'author': 'Craig Trim',
    'author_email': 'craigtrim@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '==3.8.5',
}


setup(**setup_kwargs)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-


import os

from baseblock import BaseObject

from dbpedia.redirects.dmo import DBpediaRedirectsParser


class GenerateRedirectsFile(BaseObject):
    """ Generate Entity Variations (Synonyms for Entities) """

    def __init__(self):
        """ Change Log

        Created:
            14-Jul-2022
            craigtrim@gmail.com
            *   https://github.com/craigtrim/dbpedia/issues/1
        """
        BaseObject.__init__(self, __name__)

    def _generate(self,
                  input_file: str,
                  output_file: str):

        dmo = DBpediaRedirectsParser(input_file)
        df_redirects = dmo.process()

        df_redirects.to_csv(output_file,
                            sep="\t",
                            index=False,
                            encoding="utf-8")

    def process(self) -> None:

        input_file = os.environ["REDIRECTS_INFILE"]

        if not os.path.exists(input_file):
            self.logger.error('\n'.join([
                "Redirects Input File Not Found",
                f"\tPath: {input_file}"]))
            raise ValueError

        output_file = os.path.join(os.environ["PROJECT_BASE"],
                                   os.environ["REDIRECTS_OUTFILE"])

        if os.path.exists(output_file) and self.isEnabledForWarning:
            self.logger.warning('\n'.join([
                "Replacing Existing File",
                f"\tPath: {output_file}"]))

        self._generate(input_file=input_file,
                       output_file=output_file)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-


from wikipedia import WikipediaPage

from baseblock import BaseObject
from dbpedia.lookup.dmo import SpacyDocSegmenter
from dbpedia.lookup.dmo import DBPediaContentNormalizer


class PerformPageTransform(BaseObject):
    """ for a given dbPedia page, create a JSON structure """

    def __init__(self,
                 ):
        """ Change Log

        Created:
            14-Jul-2022
            craigtrim@gmail.com
            *   https://github.com/craigtrim/dbpedia/issues/1
        """
        BaseObject.__init__(self, __name__)
        self._normalize_page = DBPediaContentNormalizer(
            segmenter=SpacyDocSegmenter().process).process

    def _normalized(self,
                    content: list or str) -> list:
        if type(content) == str:
            content = [content]

        return self._normalize_page(content)

    def _summary(self,
                 page: WikipediaPage) -> list:
        return self._normalized(page.summary)

    def _content(self,
                 page: WikipediaPage) -> list:
        return self._normalized(page.content)

    def process(self,
                title: str,
                page: WikipediaPage) -> dict:
        """_summary_

        Args:
            title (str): _description_
            page (WikipediaPage): _description_

        Returns:
            dict: _description_
        """

        def _references():
            try:
                return page.references
            except Exception as e:
                self.logger.error(e)

        def _categories():
            try:
                return page.categories
            except Exception as e:
                self.logger.error(e)

        def _sections():
            try:
                return page.sections
            except Exception as e:
                self.logger.error(e)

        def _parent_id():
            try:
                return str(page.parent_id)
            except Exception as e:
                self.logger.error(e)

        def _page_id():
            try:
                return str(page.pageid)
            except Exception as e:
                self.logger.error(e)

        def _revision_id():
            try:
                return str(page.revision_id)
            except Exception as e:
                self.logger.error(e)

        def _url():
            try:
                return page.url
            except Exception as e:
                self.logger.error(e)

        def _links():
            try:
                return page.links
            except Exception as e:
                self.logger.error(e)

        def _title():
            try:
                return page.title
            except Exception as e:
                self.logger.error(e)

        def _original_title():
            try:
                return page.original_title
            except Exception as e:
                self.logger.error(e)

        return {"key": title,
                "url": _url(),
                "title": _title(),
                "links": _links(),
                "content": self._content(page),
                "page_id": _page_id(),
                "summary": self._summary(page),
                "sections": _sections(),
                "parent_id": _parent_id(),
                "categories": _categories(),
                "references": _references(),
                "revision_id": _revision_id(),
                "original_title": _original_title()}



#!/usr/bin/env python
# -*- coding: UTF-8 -*-


from wikipedia import page as wiki_page
from wikipedia.exceptions import PageError
from wikipedia.exceptions import WikipediaException
from wikipedia.exceptions import DisambiguationError

from baseblock import BaseObject
from dbpedia.lookup.dto import WikipediaPageStatus


class DBpediaPageFinder(BaseObject):
    """ make a wire call to dbPedia to find a page 

    *** API LIMITS ***
        -   There is no hard and fast limit on read requests, but be considerate 
            and try not to take a site down. 
        -   Most system administrators reserve the right to unceremoniously block you 
            if you do endanger the stability of their site.
        -   https://www.mediawiki.org/wiki/API:Etiquette """

    __d_disambiguation = {}

    def __init__(self,
                 auto_suggest: bool = False):
        """ Change Log

        Created:
            14-Jul-2022
            craigtrim@gmail.com
            *   https://github.com/craigtrim/dbpedia/issues/1
        """
        BaseObject.__init__(self, __name__)
        self._auto_suggest = auto_suggest

        self.logger.debug('\n'.join([
            "Instantiated DBPediaPageFinder",
            f"\tAuto Suggest: {self._auto_suggest}"]))

    def _page_error(self,
                    title: str) -> tuple:
        self.logger.error('\n'.join([
            "Lookup Failed (Page Error)",
            f"\tTitle: {title}"]))

        if not title.islower():
            _title = title.lower()
            self.logger.debug('\n'.join([
                "Forced Lookup",
                f"\tOriginal Term: {title}",
                f"\tDisambiguation: {_title}"]))
            return DBpediaPageFinder(self._auto_suggest).process(_title)

        return None, WikipediaPageStatus.FAIL_PAGE_ERROR

    def _disambiguation_error(self,
                              title: str) -> tuple:
        if title in self.__d_disambiguation:
            _title = self.__d_disambiguation[title]
            self.logger.debug('\n'.join([
                "Forced Disambiguation",
                f"\tOriginal Term: {title}",
                f"\tDisambiguation: {_title}"]))
            return DBpediaPageFinder(self._auto_suggest).process(_title)

        self.logger.error('\n'.join([
            "Disambiguation Error",
            f"\tTitle: {title}"]))

        return None, WikipediaPageStatus.FAIL_DISAMBIGUATION_ERROR

    def _wikipedia_error(self,
                         title: str) -> tuple:
        self.logger.error('\n'.join([
            "Wikipedia Error",
            f"\tTitle: {title}"]))

        return None, WikipediaPageStatus.FAIL_WIKIPEDIA_ERROR

    def _key_error(self,
                   title: str) -> tuple:
        self.logger.error('\n'.join([
            "Lookup Failed (Key Error)",
            f"\tTitle: {title}"]))

        return None, WikipediaPageStatus.FAIL_KEY_ERROR

    def _os_error(self,
                  title: str) -> tuple:
        self.logger.error('\n'.join([
            "OS Error (Likely a bad Filename)",
            f"\tTitle: {title}"]))

        return None, WikipediaPageStatus.FAIL_OS_ERROR

    def process(self,
                title: str) -> tuple:

        if not title or not len(title):
            return None, WikipediaPageStatus.FAIL_INVALID_INPUT

        # if no ascii character input exists, don't bother moving forward
        if sum([ch.isascii() for ch in title]) == 0:
            return None, WikipediaPageStatus.FAIL_INVALID_INPUT

        try:

            page = wiki_page(title=title,
                             pageid=None,
                             preload=True,
                             redirect=True,
                             auto_suggest=self._auto_suggest)

            return page, WikipediaPageStatus.PASS

        except KeyError as e:
            self.logger.exception(e)
            return self._key_error(title)

        except OSError as e:
            self.logger.exception(e)
            return self._os_error(title)

        except PageError as e:
            self.logger.exception(e)
            return self._page_error(title)

        except DisambiguationError as e:
            self.logger.exception(e)
            return self._disambiguation_error(title)

        except WikipediaException as e:
            self.logger.exception(e)
            return self._wikipedia_error(title)

        return None, WikipediaPageStatus.FAIL_OTHER
